@@ -321,7 +321,6 @@ PSPMecodyn <- function(modelname = NULL, startstate = NULL, timepars = NULL, bif
     cppflags <- "-DR_PACKAGE"
     if (exists("CFLAGS")) cppflags <- paste0(cppflags, " ", get("CFLAGS"))
     if (debug) cppflags <- paste0(cppflags, " -DDEBUG=1 -g -Wall")
-    else cppflags <- paste0(cppflags, " -Wno-format-overflow")
     cppflags <- paste0(cppflags, " -I.", " -I\"", PSPMsrcdir.name, "\" -I\"", PSPMsrcdir.name, "/escbox\" ")
 
     hasfftw <- FALSE
@@ -409,7 +408,10 @@ PSPMecodyn <- function(modelname = NULL, startstate = NULL, timepars = NULL, bif
     output <- list(curvedesc = desc, curvepoints = data)
     return(output)
   }
-  else return()
+  else {
+    cat("\nComputations with ", modelname, " produced no output\n")
+    return()
+  }
 }
 
 
