@@ -26,7 +26,7 @@
     You should have received a copy of the GNU General Public License
     along with PSPManalysis. If not, see <http://www.gnu.org/licenses/>.
 
-    Last modification: AMdR - May 05, 2018
+    Last modification: AMdR - Oct 13, 2020
 ***/
 
 #define PSPMEVODYN                1
@@ -524,7 +524,7 @@ void ComputeCurve(const int argc, char **argv)
   double        oldpoint[MaxPntDim], evopntvec[MaxPntDim];
   double        Jac[MaxPntDim*MaxPntDim], JacCopy[MaxPntDim*MaxPntDim], dFdp[ParameterNr*MaxPntDim];
   double        y[MaxPntDim], rhs[MaxPntDim], pardif, oldcurvestep;
-  char          csbname[MAX_STR_LEN], errname[MAX_STR_LEN], outname[MAX_STR_LEN];
+  char          csbname[3*MAX_STR_LEN], errname[3*MAX_STR_LEN], outname[3*MAX_STR_LEN];
   char          tmpstr[MAX_STR_LEN];
   struct stat   buffer;
 
